/**
 * JXTA List Members of PeerGroup
 *
 */

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.peer.PeerID;
import net.jxta.protocol.PeerAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.exception.PeerGroupException;

import net.jxta.discovery.DiscoveryService;
import net.jxta.pipe.PipeService;
import net.jxta.membership.MembershipService;
import net.jxta.membership.Authenticator;
import net.jxta.credential.AuthenticationCredential;
import net.jxta.credential.Credential;
 
import net.jxta.resolver.ResolverService;

import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.id.IDFactory;

import net.jxta.document.StructuredTextDocument;
import net.jxta.document.MimeMediaType;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;

public class JXTAListPG {

    static final int RETRIES = 3;
    static final int TIMEOUT = 3000;

    PeerGroup gruppe = null;
    Screen sc = new Screen();
    String pgname = "InTecPeerGroup";
    String pgURL  = "jxta:uuid-3E2406ADF2E542FD87F1A9744052C30D02";
	// generated with mkpgrp xxx;

    public static void main(String[] args) {
	JXTAListPG listpg = new JXTAListPG();
	listpg.startJXTA(args);
    }

    void startJXTA(String[] args) {
	sc.println("starting JXTA List PeerGroup");
	try {
	    gruppe = PeerGroupFactory.newNetPeerGroup();
	} catch (PeerGroupException e) {
	    e.printStackTrace();
	    return;
	}
	if ( gruppe == null ) { 
           sc.println("Net Group not found, no way to continue");
           return;
	}

        PeerGroupID pgid = gruppe.getPeerGroupID();
        sc.println("pgid:  " + pgid);
	PeerID pid = gruppe.getPeerID();
        sc.println("pid:   " + pid);
	String pname = gruppe.getPeerName();
        sc.println("pname: " + pname);

	PeerGroup ipg = null;
        sc.println("searching " + pgname);
	try {
	    ipg = searchPG(pgname);
	} catch (Exception e) {
	     e.printStackTrace();
	}
	if ( ipg == null ) return;
        sc.println("searched " + pgname);

        PeerGroupID ipgid = ipg.getPeerGroupID();
        sc.println("ipgid:  " + ipgid);

	// core services:
	DiscoveryService idisco  = ipg.getDiscoveryService();
	PipeService ipipe        = ipg.getPipeService();

        PeerGroupAdvertisement adv = ipg.getPeerGroupAdvertisement();
	DiscoveryService disco = gruppe.getDiscoveryService();
        disco.remotePublish(adv, DiscoveryService.GROUP);
        sc.println(pgname + " published");

        sc.println("joining peer group " + pgname);
	try {
            joinPeerGroup( ipg );
	} catch (Exception e) {
	     e.printStackTrace();
	}
        sc.println("peer group joined");

        sc.println("discovering peers");
	try {
            discoverPeers( ipg );
            discoverPeers( gruppe );
	} catch (Exception e) {
	     e.printStackTrace();
	}
        sc.println("peers discovered");
    }

    private PeerGroup searchPG(String pgname) throws Exception {
	PeerGroup ipg = null;
	DiscoveryService disco = gruppe.getDiscoveryService();
	Enumeration pgs = null;
	for (int i = RETRIES; i > 0; i-- ){
	    try {
		pgs = disco.getLocalAdvertisements(
						   DiscoveryService.GROUP,
						   "Name",
						   pgname);
		if ( pgs != null && pgs.hasMoreElements() ) break;
		disco.getRemoteAdvertisements(
		                              null,
					      DiscoveryService.GROUP,
					      "Name",
					      pgname,
					      1,
					      null);
		try { 
		    Thread.sleep( TIMEOUT );
		} catch (InterruptedException e) { }
	    } catch (IOException e) { }
	}

	PeerGroupAdvertisement pgAdv = null;
	if ( pgs != null && pgs.hasMoreElements() ) {
            sc.println("PeerGroup " + pgname + " found");
	    try {
                 pgAdv = (PeerGroupAdvertisement) pgs.nextElement();
	         ipg = gruppe.newGroup( pgAdv );
	    } catch (PeerGroupException e) {
		e.printStackTrace();
	    }
	    return ipg;
	}

        sc.println("PeerGroup " + pgname + " not found, creating new one");

	ModuleImplAdvertisement implAdv = 
              gruppe.getAllPurposePeerGroupImplAdvertisement();
	// throws Exception sic

        ipg = gruppe.newGroup(
			      mkGroupID(),
			      implAdv,
			      pgname,
			      "The " + pgname + " Description"
                             );
	//PeerGroupAdvertisement ipgadv = ipg.getPeerGroupAdvertisement();

	return ipg;
    }

    private PeerGroupID mkGroupID() throws Exception {
	URL u = new URL( "urn", "", pgURL );
	return (PeerGroupID) IDFactory.fromURL( u );
    }

    private void discoverPeers(PeerGroup pg) throws Exception {
	sc.println("discovering peers in group " + pg.getPeerGroupName() );
	DiscoveryService disco = pg.getDiscoveryService();
	Enumeration ps = null;
	for (int i = RETRIES; i > 0; i-- ){
	    try {
                disco.getRemoteAdvertisements(
                       null, disco.PEER, null, null,2, null);
		try { 
		    Thread.sleep( TIMEOUT );
		} catch (InterruptedException e) { }
	    } catch (/*IO*/Exception e) { }
	}
        ps = disco.getLocalAdvertisements(disco.PEER, null, null);
	// sc.println("peers: " + ps);
	if ( ps == null ) return;
	while ( ps.hasMoreElements() ) {
	    try {
                 PeerAdvertisement pa = (PeerAdvertisement) ps.nextElement();
	         sc.println("found peer: " + pa.getName());
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }

    private void joinPeerGroup(PeerGroup pg) throws Exception {
	MembershipService ms = pg.getMembershipService();
	/*
	DiscoveryService disco = pg.getDiscoveryService();
	Credential c = ms.getDefaultCredential();
        sc.println("credential " + c);
	*/
        AuthenticationCredential ac = new AuthenticationCredential(pg, null, null);
	Authenticator a = ms.apply( ac );

	if ( a.isReadyForJoin() ) {
           ms.join( a );
	} else {
	    sc.println("a not ready for join "+ pg.getPeerGroupName());
	}
    }
}
